import { 
  select, 
  csv, 
  scaleLinear, 
  scaleBand, 
  max, 
  axisLeft,
  axisBottom
} from 'd3';

import { getOrderedNotes } from './orderedNotes';

const svg = select('svg');
svg.style('background-color', '#f7efef');

const width  = +svg.attr('width');
const height = +svg.attr('height');

const render = data => {
  const title = 'Notes: Midi tick X Note';
  
  const xValue = d => d.start;
  const xAxisLabel = 'Midi tick';
  
  const yValue = d => d.note;
  const circleRadius = 10;
  const yAxisLabel = 'Note';
  
  const margin = { top: 60, right: 40, bottom: 88, left: 90 };
  const innerWidth = width - margin.left - margin.right;
  const innerHeight = height - margin.top - margin.bottom;
  
  const xScale = scaleLinear()
    .domain([0, max(data, xValue)])
    .range([0, innerWidth]);
  
  const orderedNotes = getOrderedNotes(data);
  const yScale = scaleBand()
    .domain(orderedNotes)
    .range([0, innerHeight])
    .padding(0.2);
  
  const g = svg.append('g')
    .attr('transform', `translate(${margin.left}, ${margin.top})`);
  
  const xAxis = axisBottom(xScale)
    .tickSize(-innerHeight)
    .tickPadding(15);
  
  const yAxis = axisLeft(yScale)
    .tickSize(-innerWidth)
    .tickPadding(10);

  const yAxisG = g.append('g').call(yAxis);
  yAxisG.selectAll('.domain').remove();
  
  yAxisG.append('text')
      .attr('class', 'axis-label')
      .attr('y', -50)
      .attr('x', -innerHeight / 2)
      .attr('fill', 'black')
      .attr('transform', `rotate(-90)`)
      .attr('text-anchor', 'middle')
      .text(yAxisLabel);
  
  const xAxisG = g.append('g').call(xAxis)
    .attr('transform', `translate(0,${innerHeight})`);
  
  xAxisG.select('.domain').remove();
  
  xAxisG.append('text')
      .attr('class', 'axis-label')
      .attr('y', 75)
      .attr('x', innerWidth / 2)
      .attr('fill', 'black')
      .text(xAxisLabel);

  g.selectAll('circle').data(data)
    .enter().append('circle')
      .attr('cy', d => yScale(yValue(d)))
      .attr('cx', d => xScale(xValue(d)))
      .attr('r', circleRadius);
  
  g.append('text')
      .attr('class', 'title')
      .attr('y', -10)
      .text(title);

};

csv('https://vizhub.com/rubens2005/datasets/poor-little-fool_track-12.csv').then(data => {
  data.forEach(d => {
    d.start = +d.start;
    d.end = +d.end;
    d.duration = +d.duration;
  });
  render(data);
});